@@ -3,7 +3,6 @@ import datetime
 import traceback
 from typing import Union
 from django.conf import settings
-from django.db import OperationalError
 from circuit_maintenance_parser import ParsingError, init_provider
 from nautobot.circuits.models import Circuit, Provider
 from nautobot.extras.jobs import Job
@@ -183,6 +182,7 @@ def process_raw_notification(logger: Job, notification: MaintenanceNotification)
         )
         return None
 
+    raw_payload = b""
     for raw_payload in notification.raw_payloads:
         parser = init_provider(raw=raw_payload, provider_type=notification.provider_type)
         if not parser:
@@ -192,18 +192,20 @@ def process_raw_notification(logger: Job, notification: MaintenanceNotification)
         try:
             parsed_notifications = parser.process()
             break
-        except ParsingError as exc:
-            tb_str = traceback.format_exception(etype=type(exc), value=exc, tb=exc.__traceback__)
-            logger.log_debug(message=f"Parsing failed for notification {notification.subject}:.\n{tb_str}")
-        except Exception as exc:
-            tb_str = traceback.format_exception(etype=type(exc), value=exc, tb=exc.__traceback__)
+        except ParsingError:
+            tb_str = traceback.format_exc()
+            logger.log_debug(message=f"Parsing failed for notification `{notification.subject}`:\n```\n{tb_str}\n```")
+        except Exception:
+            tb_str = traceback.format_exc()
             logger.log_debug(
-                message=f"Unexpected exception while parsing notification {notification.subject}.\n{tb_str}"
+                message=f"Unexpected exception while parsing notification `{notification.subject}`.\n```\n{tb_str}\n```"
             )
     else:
         parsed_notifications = []
-        raw_payload = b""
-        logger.log_warning(message=f"Parsed failed for all the raw payloads for '{notification.subject}'.")
+        logger.log_warning(message=f"Parsed failed for all the raw payloads for `{notification.subject}`.")
+        # Carry on with the last raw_payload in the list, if any
+        if not raw_payload:
+            return None
 
     if isinstance(raw_payload, str):
         raw_payload = raw_payload.encode("utf-8")
@@ -217,7 +219,7 @@ def process_raw_notification(logger: Job, notification: MaintenanceNotification)
             sender=notification.sender,
             source=NotificationSource.objects.filter(name=notification.source).last(),
         )
-    except OperationalError as exc:
+    except Exception as exc:
         logger.log_warning(message=f"Raw notification '{notification.subject}' not created because {str(exc)}")
         return None
 
@@ -282,7 +284,7 @@ class HandleCircuitMaintenanceNotifications(Job):
                 return raw_notification_ids
 
             for notification in notifications:
-                self.log_info(message=f"Processing notification {notification.subject}.")
+                self.log_info(message=f"Processing notification `{notification.subject}`.")
                 raw_id = process_raw_notification(self, notification)
                 if raw_id:
                     raw_notification_ids.append(raw_id)
