@@ -1,5 +1,6 @@
 """Models for Circuit Maintenance."""
 import hashlib
+import pickle  # nosec
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.urls import reverse
@@ -172,6 +173,9 @@ class NotificationSource(OrganizationalModel):
         help_text="The Provider(s) that this Notification Source applies to.",
         blank=True,
     )
+    _token = models.BinaryField(
+        blank=True,
+    )
 
     csv_headers = ["name", "slug", "providers"]
 
@@ -190,6 +194,16 @@ class NotificationSource(OrganizationalModel):
         """Return fields for bulk view."""
         return (self.name, self.slug, self.providers)
 
+    @property
+    def token(self):
+        """Getter for _token."""
+        return pickle.loads(self._token)  # nosec
+
+    @token.setter
+    def token(self, value):
+        """Setter for _token."""
+        self._token = pickle.dumps(value)
+
 
 @extras_features(
     "custom_fields",
